import _Promise from 'babel-runtime/core-js/promise';
import _JSON$stringify from 'babel-runtime/core-js/json/stringify';
import _Object$keys from 'babel-runtime/core-js/object/keys';
import Axios from 'axios';
import { parseText, logger, isBoolean } from '../util';
import schemer from './schemer.js';

var axios = Axios.create();
var CancelToken = Axios.CancelToken;

function convert(sub, options) {
    options = options || {};
    var ROOT = sub.ROOT || '';
    delete sub.ROOT;

    return _Object$keys(sub).reduce(function (ctx, key) {

        var headRequest = ['get', 'delete', 'head', 'options'];
        var bodyRequest = ['post', 'put', 'patch'];
        var item = sub[key];
        var method = _Object$keys(item).filter(function (subkey) {
            if (~headRequest.concat(bodyRequest).indexOf(subkey)) {
                return true;
            }
        })[0];

        var rawUrl = item[method];
        // console.log(key,rawUrl)

        // !item.payload && (item.payload = true)
        item.payload = isBoolean(item.payload) ? item.payload : isBoolean(options.payload) ? options.payload : true;

        var type = item.payload ? 'json' : 'form';
        var abort = item.abort || false;
        var binary = item.binary || false;
        var response = item.response || false;
        var schema = item.schema || false;
        var withCredentials = item.withCredentials || false;
        var compile = typeof item.compile == 'boolean' ? item.compile : true;
        var mock = item.mock || false;
        // console.log(item)

        var contentType = {
            'json': 'application/json',
            'form': 'application/x-www-form-urlencoded'
        }[type];

        ctx[key] = function (parse) {
            var data = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
            var config = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

            var pend = null;
            var url = compile ? parseText(rawUrl, parse) : rawUrl;

            logger('frame', '--- %s', rawUrl);
            logger('frame', '+++ %s method:%s', url, method);

            if (compile && !rawUrl.match(/\{(.+?)\}/)) {
                if (config) {
                    logger('frame', '~url参数%s貌似无用', _JSON$stringify(parse));
                } else {
                    config = data;
                    data = parse;
                    parse = null;
                }
            }
            if (!compile) {
                config = data;
                data = parse;
                parse = null;
            }

            if (~headRequest.indexOf(method)) {
                config = data;
                data = null;
                config = expandHeaders({ config: config, contentType: contentType, apiname: key, baseURL: ROOT, schema: schema, abort: abort, binary: binary, withCredentials: withCredentials, mock: mock });
                pend = axios[method](url, config);
            }

            if (~bodyRequest.indexOf(method)) {
                config = expandHeaders({ config: config, contentType: contentType, apiname: key, baseURL: ROOT, schema: schema, abort: abort, binary: binary, withCredentials: withCredentials, mock: mock });
                // console.log(data,type)
                // if(data){}
                type == 'form' && (data = data && serialize(data));

                pend = axios[method](url, data, config);
            }

            // return pend

            pend.catch(function (err) {
                console.log({ err: err });
            });

            return {
                then: function then(res, rej) {
                    rej = rej || function () {};
                    return pend.then(res, rej);
                },
                catch: function _catch(capture) {
                    capture = capture || function () {};
                    return pend.catch(capture);
                }
            };
        };
        return ctx;
    }, {});
}

function expandHeaders(params) {
    var config = params.config,
        contentType = params.contentType,
        apiname = params.apiname,
        baseURL = params.baseURL,
        schema = params.schema,
        abort = params.abort,
        binary = params.binary,
        withCredentials = params.withCredentials,
        mock = params.mock;


    config = config || {};
    config.headers = config.headers || {};
    config.headers['content-type'] = contentType;

    abort && (config.cancelToken = new CancelToken(function (cancel) {
        config.$$abort = cancel;
    }));

    config.$$apiname = apiname;
    config.$$schema = schema;
    baseURL && (config.baseURL = baseURL);
    withCredentials && (config.withCredentials = withCredentials);
    binary && (config.responseType = 'arraybuffer');
    mock && (config.headers['mock-remote'] = mock);

    // console.log(config)
    return config;
}

function serialize(data) {
    return _Object$keys(data).map(function (key) {
        return key + '=' + data[key];
    }).join('&');
}

function interceptors() {
    var requests = {};
    axios.interceptors.request.use(function (config) {
        var apiname = config.$$apiname;
        var request = requests[apiname] = requests[apiname] || [];

        if (request.length) {
            // console.log(requests, apiname)
            request.shift().abort('~ repeat request ' + config.$$apiname + ' is abort');
        }
        config.$$abort && request.push({ abort: config.$$abort });
        return config;
    }, function (error) {
        return _Promise.reject(error);
    });

    axios.interceptors.response.use(function (response) {
        // console.log(response.config)
        var schema = response.config.$$schema;

        schema && schemer(schema, response.data, response.config.url);

        // console.log(response.data)
        return response;
    });
}
interceptors();

export default convert;
export { axios };