import logger from './logger.js';
import { isUndefined } from './lang.js';

function parseURL(url) {
  var parse_url = /^(?:([A-Za-z]+):)?(\/{0,3})([0-9._\-A-Za-z]+)(?::(\d+))?(?:\/([^?#]*))?(?:\?([^#]*))?(?:#(.*))?$/;
  var result = parse_url.exec(url);
  return ['url', 'scheme', 'slash', 'host', 'port', 'path', 'query', 'hash'].reduce(function (obj, field, index) {
    var ret = result[index];
    if ('path' == field) {
      ret = ret ? '/' + ret : '';
    }
    obj[field] = ret;
    return obj;
  }, {});
}

/**
 * [parseText 编译标示符]
 * @param  {[type]} rawURL [/talkinggenius/?page={page}&size={size}&templateId={templateId:}]
 * @param  {[type]} obj    [{page:10,size:20}]
 * @return {[type]}        [/talkinggenius/?page=10&size=20]
 */
function parseText(rawURL, obj) {
  var url = compilerText(rawURL, obj);
  var ourl = parseURL(url);
  // console.log(ourl.host,ourl.path,validURLQuery(ourl.query))
  var urlArr = [ourl.host, ourl.path];
  var query = validURLQuery(ourl.query);
  if (query) urlArr = urlArr.concat(['?', validURLQuery(ourl.query)]);

  return urlArr.join('');
}

/**
 * [compiler 编译文本]
 * @param  {[type]} text [v1classes{userid}audio-upload]
 * @param  {[type]} obj  [{userid:aaa}]
 * @return {[type]}      [v1classesaaaaudio-upload]
 */
function compilerText(raw, obj) {

  obj = obj || {};
  var ripe = raw.replace(/(\{.*?\})/g, function () {
    for (var _len = arguments.length, arg = Array(_len), _key = 0; _key < _len; _key++) {
      arg[_key] = arguments[_key];
    }

    var key = arg[0].replace(/[\{|\}]/g, '').trim(),
        pair = void 0,
        defaulter = void 0,
        val = void 0;

    pair = key.split(':');
    key = pair[0];
    defaulter = pair[1];
    val = obj[key];
    val = !isUndefined(val) ? val : defaulter;

    if (isUndefined(val)) {
      logger('error', '\u5728' + raw + '\u4E2D,\u53C2\u6570' + key + '\u672A\u8D4B\u503C');
    }

    // if(isUndefined(val)) {
    //   val = isUndefined(defaulter) ? `{${keyArr[0]}}` : defaulter
    // }

    return val;
  });
  return ripe;
}

/**
 * [validURLQuery 有效 url query]
 * @param  {[type]} query [url]
 * @return {[type]}       [valid query]
 */
function validURLQuery(query) {
  return query && query.split('&').filter(function (item) {
    var arr = item.split('=');
    if (arr[1]) return true;
  }).join('&');
}

var compilerURL = parseText;

export { parseURL, compilerText, validURLQuery, parseText, compilerURL };